@@ -10,6 +10,7 @@ from urllib.parse import parse_qs
 
 import xmltodict
 from flask import Response as FlaskResponse
+from moto.core.exceptions import JsonRESTError
 from requests.models import CaseInsensitiveDict
 from requests.models import Response as RequestsResponse
 
@@ -40,6 +41,16 @@ class ErrorResponse(Exception):
         self.response = response
 
 
+class ResourceNotFoundException(JsonRESTError):
+    """Generic ResourceNotFoundException used when processing requests in Flask contexts."""
+
+    code = 404
+
+    def __init__(self, message=None):
+        message = message or "The given resource cannot be found"
+        super(ResourceNotFoundException, self).__init__("ResourceNotFoundException", message)
+
+
 def flask_error_response_json(
     msg: str, code: Optional[int] = 500, error_type: Optional[str] = "InternalFailure"
 ):
