@@ -30,7 +30,6 @@ from localstack.utils.aws import templating
 from localstack.utils.aws.aws_models import KinesisStream
 from localstack.utils.common import (
     get_service_protocol,
-    is_port_open,
     is_string,
     is_string_or_bytes,
     make_http_request,
@@ -255,16 +254,6 @@ def get_local_service_url(service_name_or_port):
     return os.environ["TEST_%s_URL" % service_name_upper]
 
 
-def is_service_enabled(service_name):
-    """Return whether the service with the given name (e.g., "lambda") is available."""
-    try:
-        url = get_local_service_url(service_name)
-        assert url
-        return is_port_open(url, http_path="/", expect_success=False)
-    except Exception:
-        return False
-
-
 def connect_to_resource(
     service_name, env=None, region_name=None, endpoint_url=None, *args, **kwargs
 ):
@@ -546,6 +535,7 @@ def get_iam_role(resource, env=None):
     return "role-%s" % resource
 
 
+# TODO: remove this (can't statically define secret ARN because it includes a random suffix)
 def secretsmanager_secret_arn(secret_id, account_id=None, region_name=None):
     if ":" in (secret_id or ""):
         return secret_id
